@@ -112,7 +112,6 @@ static int ctBcsReset(HANDLE hDevice,unsigned char *ATR,unsigned char *ATR_len,u
   int error;
   int warm;
   int protocol=0;
-  int Res;
   *hist_len=0;
   hist=ATR;
   warm=0;
@@ -265,6 +264,11 @@ static int ctBcsReset(HANDLE hDevice,unsigned char *ATR,unsigned char *ATR_len,u
   {
      int in_len;
      unsigned char rbuffer[5];
+	  if((TA1 & 0x0f)==0x06) // D factor of 32 is to fast
+	  {
+		  if(TA1!=0x96 || (protocol&1) || TC1==0xff)
+		     TA1=(TA1 & 0xf0) | 0x05;
+	  }
      CJPP_TEST(cjccid_SetParameters(hDevice,(unsigned char)((protocol&1)?0:1),TA1,0,TC1,(unsigned char)((protocol&1)?TC2:TB3)));
      if(specific==0 && hasTA1)
      {
@@ -2011,70 +2015,39 @@ static char ctapiData_inner(CCID_DEVICE hDevice,unsigned char *dad,unsigned char
                         PinType=1;
                         PinLengthPosition=4;
                         PinLengthSize=4;
-                        PinPosition=1;
-                        if(taglen==7)
-                        {
-                           if(PinLength==0)
-                           {
-                              Min=4;
-                              Max=12;
-                              Condition=2;
-                              PinLength=7;
-                              buffer[4]=0x08;
-                              memset(buffer+5,0xff,8);
-                              buffer[5]=0x2f;
-                              outlen=13;
-                           }
-                           else
-                           {
-                              Min=PinLength;
-                              Max=PinLength;
-                              if(PinLength<4 || PinLength>12)
-                              {
-                                 response[0]=0x67;
-                                 response[1]=0x00;
-                                 *lenr=2;
-                                 return 0;
-                              }
-                              PinLength=7;
-                              Condition=1;
-                              buffer[4]=0x08;
-                              memset(buffer+5,0xff,8);
-                              buffer[5]=0x2f;
-                              outlen=13;
-                           }
-                        }
-                        else if(buffer[4]!=8)
+                        PinPosition++;
+								if(PinPosition>240)
                         {
                            response[0]=0x67;
                            response[1]=0x00;
                            *lenr=2;
                            return 0;
+                        }
+								if(buffer[4]<PinPosition+7)
+									buffer[4]=PinPosition+7;
+                        buffer[4+PinPosition]=0x2f;
+                        memset(buffer+5+PinPosition,0xff,7);
+                        outlen=buffer[4]+5;
+                        if(PinLength==0)
+                        {
+                           Min=4;
+                           Max=12;
+                           Condition=2;
+                           PinLength=7;
                         }
                         else
                         {
-                           if(PinLength==0)
+                           Min=PinLength;
+                           Max=PinLength;
+                           if(PinLength<4 || PinLength>12)
                            {
-                              Min=4;
-                              Max=12;
-                              Condition=2;
-                              PinLength=7;
-                              outlen=taglen-2;
-                           }
-                           else
-                           {
-                              if(PinLength<4 || PinLength>12)
-                              {
-                                 response[0]=0x67;
-                                 response[1]=0x00;
-                                 *lenr=2;
-                                 return 0;
-                              }
-                              Min=PinLength;
-                              Max=PinLength;
-                              Condition=1;
-                              outlen=taglen-2;
+                              response[0]=0x67;
+                              response[1]=0x00;
+                              *lenr=2;
+                              return 0;
                            }
+                           PinLength=7;
+                           Condition=1;
                         }
                         break;
                      default:
@@ -2408,76 +2381,49 @@ static char ctapiData_inner(CCID_DEVICE hDevice,unsigned char *dad,unsigned char
                         break;
                      case 2:
                         PinType=1;
-                        PinPosition=1;
+                        PinPositionNew++;
+                        PinPositionOld++;
                         PinLengthPosition=4;
                         PinLengthSize=4;
-                        if(taglen==8)
-                        {
-                           if(PinLength==0)
-                           {
-                              Min=4;
-                              Max=12;
-                              Condition=2;
-                              PinLength=7;
-                              buffer[4]=16;
-                              memset(buffer+5,0xff,16);
-                              buffer[5]=0x2f;
-                              buffer[13]=0x2f;
-                              outlen=21;
-                              PinPositionOld=0;
-                              PinPositionNew=8;
-                           }
-                           else
-                           {
-                              Min=PinLength;
-                              Max=PinLength;
-                              if(PinLength<4 || PinLength>12)
-                              {
-                                 response[0]=0x67;
-                                 response[1]=0x00;
-                                 *lenr=2;
-                                 return 0;
-                              }
-                              PinLength=7;
-                              Condition=1;
-                              buffer[4]=16;
-                              memset(buffer+5,0xff,16);
-                              buffer[5]=0x2f;
-                              buffer[13]=0x2f;
-                              outlen=13;
-                           }
-                        }
-                        else if(buffer[4]!=16)
+
+								if(PinPositionNew>240 || PinPositionOld>240)
                         {
                            response[0]=0x67;
                            response[1]=0x00;
                            *lenr=2;
                            return 0;
+                        }
+								if(buffer[4]<PinPositionOld+7)
+									buffer[4]=PinPositionOld+7;
+								if(buffer[4]<PinPositionNew+7)
+									buffer[4]=PinPositionNew+7;
+                        buffer[4+PinPositionOld]=0x2f;
+                        memset(buffer+5+PinPositionOld,0xff,7);
+                        buffer[4+PinPositionNew]=0x2f;
+                        memset(buffer+5+PinPositionNew,0xff,7);
+
+                        outlen=buffer[4]+5;
+
+                        if(PinLength==0)
+                        {
+                           Min=4;
+                           Max=12;
+                           Condition=2;
+                           PinLength=7;
                         }
                         else
                         {
-                           if(PinLength==0)
+                           Min=PinLength;
+                           Max=PinLength;
+                           if(PinLength<4 || PinLength>12)
                            {
-                              Min=4;
-                              Max=12;
-                              Condition=2;
-                              PinLength=7;
-                              outlen=taglen-3;
-                           }
-                           else
-                           {
-                              if(PinLength<4 || PinLength>12)
-                              {
-                                 response[0]=0x67;
-                                 response[1]=0x00;
-                                 *lenr=2;
-                                 return 0;
-                              }
-                              Min=PinLength;
-                              Max=PinLength;
-                              Condition=1;
-                              outlen=taglen-3;
+                              response[0]=0x67;
+                              response[1]=0x00;
+                              *lenr=2;
+                              return 0;
                            }
+                           PinLength=7;
+                           Condition=1;
                         }
                         break;
                      default:
