@@ -55,6 +55,7 @@ static void usage( char *argv0 )
 	printf("Usage: %s [-l lib] [-p num]\n",argv0);
 	printf("   -l lib    CT-API library, default is libctapi.so.\n");
 	printf("   -p num    Port number (1-n), default is 1.\n");
+	printf("   -c count  Number of consecutive reads, default is 1.\n");
 	exit(1);
 }
 
@@ -150,12 +151,9 @@ err_sad:
 	return 3;
 }
 
-static int test_ctapi( char *argv0, int pn )
+static int test_ctapi( char *argv0, int pn, int count )
 {
-	IU8 cmd[5+255+1], rsp[256+2];
-	IU8 sad, dad;
 	IU16 ctn=1;
-	IU16 lenc, lenr=256+2;
 	IS8 ret;
 
 	int i;
@@ -167,14 +165,13 @@ static int test_ctapi( char *argv0, int pn )
 		return 3;
 	}
 
-	for (i = 0; i < 5; i++) {
+	for (i = 0; i < count; i++) {
 		ret = read_geldkarte(ctn);
 		if (ret) {
 			CT_close(ctn);
 			return ret;
 		}
 	}
-
 	CT_close(ctn);
 }
 
@@ -183,9 +180,10 @@ int main(int argc, char **argv)
 	IU16 pn=1;
 	int c;
 	int ret;
+	int count = 1;
 	char *ctlib="libctapi.so";
 
-	while ((c = getopt(argc, argv, "l:p:") != -1) ) {
+	while ((c = getopt(argc, argv, "l:p:c:")) != -1) {
 		switch (c) {
 		case 'l':
 			ctlib = malloc(strlen(optarg));
@@ -199,6 +197,9 @@ int main(int argc, char **argv)
 			pn = atoi(optarg);
 			if (pn > 0) 
 				break;
+		case 'c':
+			count = atoi(optarg);
+			break;
 		default:
 			printf("invalid option `%c' (%d)\n", c, c);
 			usage( argv[0] );
@@ -234,7 +235,7 @@ int main(int argc, char **argv)
 	if( (ret=test_cjopen(argv[0],pn))!=0 ) return ret;
 #endif
 
-	ret = test_ctapi(argv[0],pn);
+	ret = test_ctapi(argv[0], pn, count);
 
 	return ret;
 }
